import "source-map-support/register"
export type DependencyMatcher = string | RegExp
export class DependencyManager {
    private context: { [index: string]: DependencyMatcher[] } = {}
    add(name: string, deps: DependencyMatcher[]): void {
        if (this.context[name]) throw new Error(`Duplicate dependency entry: ${name}`)
        if (deps && deps.length == 0) deps = null
        this.context[name] = deps
    }
    resolveDependency(name: string, chain: string[] = []): string[] {
        if (!this.context[name]) return []
        chain = chain.slice()
        if (chain.indexOf(name) >= 0) {
            chain.push(name)
            throw new Error(`Recursive dependency detected: ${chain.join("\n=>")}`)
        }
        chain.push(name)
        let result: string[] = []
        let all: string[] = []
        for (let matcher of this.context[name] || []) {
            if (typeof matcher === "string") {
                if (all.indexOf(matcher) < 0) all.push(matcher)
            } else if (matcher instanceof RegExp) {
                for (let name in this.context) {
                    if (matcher.test(name) && all.indexOf(name) < 0) {
                        all.push(name)
                    }
                }
            }
        }
        for (let dep of all) {
            let deps = this.resolveDependency(dep, chain)
            for (let subDep of deps) {
                if (result.indexOf(subDep) < 0) result.push(subDep)
            }
            result.push(dep)
        }
        return result
    }
    resolve(): string[] {
        let result: string[] = []
        for (let name in this.context) {
            let deps = this.resolveDependency(name)
            for (let dep of deps) {
                if (result.indexOf(dep) < 0) result.push(dep)
            }
            if (result.indexOf(name) < 0) result.push(name)
        }
        return result
    }
}
