import { Service } from "./service";
import * as express from "express";
export interface Request extends express.Request {
}
export interface Response extends express.Response {
}
export declare abstract class RouteService extends Service {
    routes: RouteConstructor[];
    prefix: string;
    addRoute(RouteConstructor: any): void;
    installTo(server: express.Application): void;
    addRoutes(...obj: any[]): void;
    isRouteConstructor(obj: any): boolean;
}
export interface RouteConstructor {
    new (): Route<any>;
}
export declare abstract class Route<TMIME extends MIME> {
    constructor();
    name: string;
    mime: TMIME;
    abstract path: string;
    method: METHOD_ENUM;
    params: {};
    body: {};
    services: {
        [index: string]: Service;
    };
    query: {};
    next: Function;
    req: Request;
    res: Response;
    handleRawRequest(req: any, res: any, next: any): void;
    end(content: any): void;
    abstract handle(): any;
}
export declare abstract class DynamicResourceRoute<T extends MIME> extends Route<T> {
}
export declare abstract class MiddlewareRoute extends Route<never> {
    path: any;
    method: Method.ALL;
    mock(url: string, data: {
        [name: string]: any;
    }, callback: (err?: any) => void): void;
}
export declare abstract class PageRoute extends Route<"text/html"> {
    method: Method.GET;
    readonly mime: string;
    html(content: string): void;
}
export declare abstract class APIRoute<TResponse> extends Route<"application/json"> {
    success(data: TResponse): void;
    error(err: Error): void;
    _error: (err: Error) => void;
    _success: (data: any) => void;
    json(data: any): void;
    mock(url: string, data: {
        [name: string]: any;
    }, callback: (err?: any, data?: TResponse) => void): void;
}
export declare type MIME = "text/html" | "text/javascript" | "text/json" | "application/json" | string;
export declare type METHOD_ENUM = "GET" | "POST" | "DELETE" | "PUT" | "ALL";
export declare type Method<T extends METHOD_ENUM> = string & T;
export declare namespace Method {
    const GET: Method<"GET">;
    const POST: Method<"POST">;
    const DELETE: Method<"DELETE">;
    const PUT: Method<"PUT">;
    const ALL: Method<"ALL">;
    type POST = Method<"POST">;
    type GET = Method<"GET">;
    type DELETE = Method<"DELETE">;
    type PUT = Method<"PUT">;
    type ALL = Method<"ALL">;
}
